package com.unitesk.requality.eclipse.tools;

import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Scanner;
import java.util.UUID;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.json.JSONException;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.IStorageDeltaListener;
import com.unitesk.requality.core.NodeMapper;
import com.unitesk.requality.core.TreeStorage;

public class ResourceStorage extends TreeStorage {
    private static final String ROOT_NODE_FILENAME = "root.json";
    private Map<IFile, UUID> resourceToNode = new HashMap<IFile, UUID>();
    private IProject project;

    private IResourceChangeListener deltaListener = new DeltaProcessor(
            resourceToNode, this);

    public IResourceChangeListener getResourceChangeListener() {
        return deltaListener;
    }

    public ResourceStorage(IProject project) {
        this.project = project;
    }

    public IFile generateAttributeResourceIFile(UUID uuid, String filename) {
        IFile result = null;
        try {
            IFolder resFolder = getResourceFolder(uuid).getFolder(
                    new Path(getNodeId(uuid) + "_resources"));
            if (resFolder.exists()) {
                resFolder.create(false, true, null);
            }
            result = resFolder.getFile(filename);
        } catch (CoreException e) {
            e.printStackTrace();
        }

        return result;
    }

    @Override
    public UUID getRootNode() {
        IFile file = project.getFile(ROOT_NODE_FILENAME);
        if (file.exists()) {
            if (resourceToNode.get(file) == null) {
                UUID uuid = getUUId(file);
                resourceToNode.put(file, uuid);
            }
            return resourceToNode.get(file);
        }
        return null;
    }

    public IFolder getResourceFolder(UUID uuid) {
        String id = getNodeId(uuid);
        IFolder resFolder = getResource(uuid).getParent().getFolder(
                new Path(id + "_resources"));
        return resFolder;
    }

    @Override
    public String getNodeId(UUID uuid) {
        if (uuid.equals(getRootNode())) {
            return "/";
        }
        if (getResource(uuid) == null) {
            return null; // node not found
        }
        String jext = getResource(uuid).getFileExtension();
        String fileName = getResource(uuid).getName();
        return fileName.substring(0, fileName.length() - (jext.length() + 1));
    }

    @Override
    public void createNode(final UUID parentUUId, final UUID uuid,
            final String id, final Collection<Attribute> attrs) {
        IWorkspaceRunnable createJob = new IWorkspaceRunnable() {
            @Override
            public void run(IProgressMonitor monitor) throws CoreException {
                try {
                    IFile file;
                    if (parentUUId == null) {
                        file = project.getFile(ROOT_NODE_FILENAME);
                    } else {
                        String parentId = getNodeId(parentUUId);
                        if (parentId.equals("/")) {
                            parentId = "root";
                        }
                        IFolder childFolder;
                        if (getResource(parentUUId).getParent().getType() == IResource.PROJECT) {
                            childFolder = ((IProject) getResource(parentUUId)
                                    .getParent()).getFolder(parentId);
                        } else {
                            childFolder = ((IFolder) getResource(parentUUId)
                                    .getParent()).getFolder(parentId);
                        }
                        if (!childFolder.exists()) {
                            childFolder.create(false, true, null);
                        }
                        file = childFolder.getFile(id + ".json");
                    }

                    String jsonString = NodeMapper.toJSON(uuid, attrs)
                            .toString(4);
                    ByteArrayInputStream in = new ByteArrayInputStream(
                            jsonString.getBytes("UTF-8"));
                    file.create(in, false, null);
                    if (!file.getCharset().equals("UTF-8")) {
                        file.setCharset("UTF-8", null);
                    }
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        };

        try {
            ResourcesPlugin.getWorkspace().run(createJob, null);
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    @Override
    public Collection<Attribute> getAttributes(UUID uuid) {
        NodeMapper m = new NodeMapper(readFile(getResource(uuid)));
        return m.getAttributes();
    }

    @Override
    public UUID getParent(UUID uuid) {
        if (getResource(uuid) == null) {
            return null; // node not found
        }
        return getParent(getResource(uuid));
    }

    public UUID getParent(IFile file) {
        if (file.getParent().getType() == IResource.PROJECT) {
            return null;
        }
        String folderName = file.getParent().getName();
        String parentId = folderName.substring(0, folderName.length());

        IFile parentResource;
        IContainer parentFolder = file.getParent().getParent();
        parentResource = parentFolder.getFile(new Path(parentId + ".json"));
        if (!parentResource.exists()) {
            if (resourceToNode.containsKey(parentResource)) {
                return resourceToNode.get(parentResource);
            }
            return null;
        }
        return getUUId(parentResource);
    }

    public UUID getUUId(IFile file) {
        return new NodeMapper(readFile(file)).getUUId();
    }

    @Override
    public Collection<UUID> getChildren(UUID uuid) {
        Collection<UUID> result = new HashSet<UUID>();
        if (getResource(uuid) == null) {
            return result; // node not found
        }

        IFolder childrenFolder;

        if (getRootNode().equals(uuid)) {
            IProject project = (IProject) getResource(uuid).getParent();
            childrenFolder = project.getFolder("root");
        } else {
            String nodeId = getNodeId(uuid);
            IFolder nodeFolder = (IFolder) getResource(uuid).getParent();
            childrenFolder = nodeFolder.getFolder(nodeId);
        }

        if (!childrenFolder.exists()) {
            return result;
        }

        try {
            for (IResource res : childrenFolder.members()) {
                if (res.getType() == IResource.FILE
                        && res.getName().endsWith(".json")) {
                    UUID childUUID = getUUId((IFile) res);
                    result.add(getUUId((IFile) res));
                    resourceToNode.put((IFile) res, childUUID);
                }
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public boolean canLoad(UUID uuid) {
        return resourceToNode.containsValue(uuid);
    }

    private String readFile(IFile file) {
        try {
            if (file.getFileExtension() == null
                    || !file.getFileExtension().equals("json")) {
                return null;
            }
            StringBuilder jsonsb = new StringBuilder();
            Scanner s = new Scanner(file.getContents(), file.getCharset());
            while (s.hasNextLine()) {
                jsonsb.append(s.nextLine().trim());
            }
            s.close();
            return jsonsb.toString();
        } catch (CoreException e) {
            e.printStackTrace();
        }
        return null;
    }

    public IFile getResource(UUID uuid) {
        for (IFile r : resourceToNode.keySet()) {
            if (resourceToNode.get(r).equals(uuid))
                return r;
        }
        return null;
    }

    @Override
    public void changeNodeId(UUID uuid, String id) {
        try {
            String nodeId = getNodeId(uuid);
            IFile nodeFile = getResource(uuid);
            IPath newPath = nodeFile.getParent().getFullPath();
            newPath = newPath.addTrailingSeparator();
            IPath newFilePath = newPath.append(id + ".json");
            nodeFile.move(newFilePath, false, null);

            IFolder childrenFolder = nodeFile.getParent().getFolder(
                    new Path(nodeId));
            if (childrenFolder.exists()) {
                childrenFolder.move(newPath.append(id), false, null);
            }

        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void remove(final UUID uuid) {
        final IFile res = getResource(uuid);
        IWorkspaceRunnable deleteJob = new IWorkspaceRunnable() {
            @Override
            public void run(IProgressMonitor monitor) throws CoreException {
                String id = getNodeId(uuid);
                IFolder children = res.getParent().getFolder(new Path(id));
                IFolder resources = res.getParent().getFolder(
                        new Path(id + "_resources"));
                IContainer parent = res.getParent();
                if (resources.exists()) {
                    resources.delete(false, null);
                }
                if (children.exists()) {
                    children.delete(false, null);
                }
                res.delete(false, null);
                if (parent instanceof IFolder && parent.members().length == 0) {
                    parent.delete(false, null);
                }
            }
        };
        try {
            ISchedulingRule rule = ResourcesPlugin.getWorkspace()
                    .getRuleFactory().deleteRule(res.getParent());
            ResourcesPlugin.getWorkspace().run(deleteJob, rule, 0, null);
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void moveNodeTo(final UUID node, final UUID newParent) {
        IWorkspaceRunnable moveJob = new IWorkspaceRunnable() {
            @Override
            public void run(IProgressMonitor monitor) throws CoreException {
                String newParentNodeId = getNodeId(newParent);
                String nodeId = getNodeId(node);
                if (newParentNodeId.equals("/")) {
                    newParentNodeId = "root";
                }
                IFile nodeFile = getResource(node);
                IFile newParentFile = getResource(newParent);

                IFolder newFolder = newParentFile.getParent().getFolder(
                        new Path(newParentNodeId));
                if (!newFolder.exists()) {
                    newFolder.create(false, true, null);
                }
                IPath newPath = newFolder.getFullPath();
                newPath = newPath.addTrailingSeparator();
                IPath newFilePath = newPath.append(nodeFile.getName());

                IFolder nodeChildrenFolder = nodeFile.getParent().getFolder(
                        new Path(getNodeId(node)));
                IFolder resFolder = nodeFile.getParent().getFolder(
                        new Path(getNodeId(node) + "_resources"));

                IFolder oldNodeFolder = (IFolder) nodeFile.getParent();
                nodeFile.move(newFilePath, false, null);

                if (nodeChildrenFolder.exists()) {
                    if (nodeChildrenFolder.members().length > 0) {
                        nodeChildrenFolder.move(newPath.append(nodeId), false,
                                null);
                    } else {
                        nodeChildrenFolder.delete(false, null);
                    }
                }

                if (resFolder.exists()) {
                    if (resFolder.members().length > 0) {
                        resFolder.move(newPath.append(resFolder.getName()),
                                false, null);
                    } else {
                        resFolder.delete(false, null);
                    }
                }

                if (oldNodeFolder.members().length == 0) {
                    nodeFile.getParent().delete(false, null);
                }
            }
        };
        try {
            // TODO:
            // IResource destination;
            // IResource source;
            // ISchedulingRule rule =
            // ResourcesPlugin.getWorkspace().getRuleFactory().moveRule(source,
            // destination);
            // ResourcesPlugin.getWorkspace().run(moveJob, rule, 0, null);
            ResourcesPlugin.getWorkspace().run(moveJob, null);
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void saveAttributes(UUID uuid, Collection<Attribute> attributes) {
        try {
            String jsonString = NodeMapper.toJSON(uuid, attributes).toString(4);
            ByteArrayInputStream in = new ByteArrayInputStream(
                    jsonString.getBytes("UTF-8"));
            IFile file = getResource(uuid);
            file.setContents(in, true, true, null);
        } catch (CoreException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    @Override
    public String getNodeResourcePath(UUID uuId) {
        return getResourceFolder(uuId).getLocation().toOSString();
    }

    public IProject getProject() {
        return project;
    }

    @Override
    protected IStorageDeltaListener getDeltaTarget() {
        return super.getDeltaTarget();
    }
}
