/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package com.unitesk.requality.eclipse.wizards;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.ibm.icu.text.CharsetDetector;
import com.ibm.icu.text.CharsetMatch;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.views.TreesContentProvider;
import com.unitesk.requality.eclipse.views.actions.documents.ImportAction;
import com.unitesk.requality.eclipse.views.documents.DocLabelProvider;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;

public class DocumentImportWizardPage extends WizardPage {

	private TreeViewer docFolder;
	private Combo filenameCombo = null;
	private Combo encCombo = null;

	private String[] filenames = null;
	private Map<String, String> encoding = new HashMap<String, String>();
	private TreeDB selectedDB;

	private static String[] exts = { "*.htm;*.html;*.xhtml", "*.*" };
	private static String[] extNames = { "Markup files", "All files" };
	private static final String LBL_FILE = "Import file:";
	private static final String LBL_ENCODING = "Encoding:";
	private static final String LBL_DESTINATION = "Into folder:";

	private static final String BTN_FILE = "Browse";

	private static String TITLE = "Import";
	private static Image ICON = PlatformUI.getWorkbench().getSharedImages()
			.getImage(ISharedImages.IMG_OBJ_ELEMENT);

	protected DocumentImportWizardPage() {
		super("Document import");
	}

	@Override
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		Layout layout = new GridLayout(2, false);
		container.setLayout(layout);

		Label fileLabel = new Label(container, SWT.NONE);
		fileLabel.setText(LBL_FILE);

		Composite fileContainer = new Composite(container, SWT.NONE);
		fileContainer.setLayout(layout);
		filenameCombo = new Combo(fileContainer, SWT.BORDER);
		filenameCombo.setLayoutData(new GridData(400, -1));
		filenameCombo.addSelectionListener(fileSelect);
		filenameCombo.addModifyListener(fileRename);
		Button browseButton = new Button(fileContainer, SWT.NONE);
		browseButton.setText(BTN_FILE);
		browseButton.addSelectionListener(browseHit);

		Label encLabel = new Label(container, SWT.NONE);
		encLabel.setText(LBL_ENCODING);
		Composite encContainer = new Composite(container, SWT.NONE);
		encContainer.setLayout(layout);
		encCombo = new Combo(encContainer, SWT.READ_ONLY);
		encCombo.setLayoutData(new GridData());
		for (String csName : Charset.availableCharsets().keySet())
			encCombo.add(csName);
		encCombo.addSelectionListener(encodingSelect);

		selectEncoding("UTF-8");

		Label docFolderLabel = new Label(container, SWT.NONE);
		docFolderLabel.setText(LBL_DESTINATION);
		docFolder = new TreeViewer(container);
		GridData dfLayoutData = new GridData(GridData.FILL_BOTH);
		dfLayoutData.horizontalSpan = 2;
		docFolder.getControl().setLayoutData(dfLayoutData);

		docFolder.setContentProvider(new TreesContentProvider(Document
				.getTypeRootQId()) {
			@Override
			public Object[] getChildren(Object parentElement) {
				List<Object> normalResult = Arrays.asList(super
						.getChildren(parentElement));
				List<Object> result = new ArrayList<Object>();
				for (Object o : normalResult) {
					if (!(o instanceof Document)) {
						result.add(o);
					}
				}
				return result.toArray();
			}

			@Override
			public boolean hasChildren(Object parentElement) {
				return getChildren(parentElement).length != 0;
			}
		});

		docFolder.setLabelProvider(new DocLabelProvider());
		docFolder.setInput(ResourcesPlugin.getWorkspace().getRoot());
		docFolder.setSorter(new ViewerSorter());
		docFolder
				.addPostSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(SelectionChangedEvent event) {
						changeListener.handleEvent(new Event());
					}
				});
		docFolder.getControl().addListener(SWT.Modify, changeListener);

		this.getShell().setText(TITLE);
		this.getShell().setImage(ICON);

		setControl(parent);
	}

	private Listener changeListener = new Listener() {
		@Override
		public void handleEvent(Event event) {
			DocumentImportWizardPage.this.getContainer().updateButtons();
		}
	};

	private final SelectionListener browseHit = new SelectionListener() {
		public void widgetDefaultSelected(SelectionEvent e) {
		}

		public void widgetSelected(SelectionEvent e) {
			FileDialog fileSelect = new FileDialog(getShell(), SWT.MULTI);
			if (ImportAction.initalPath != null) {
				fileSelect.setFilterPath(ImportAction.initalPath);
			}
			fileSelect.setFileName(filenameCombo.getText());
			fileSelect.setFilterExtensions(exts);
			fileSelect.setFilterNames(extNames);
			if (fileSelect.open() != null) {
				for (String fileName : fileSelect.getFileNames()) {
					fileName = fileSelect.getFilterPath() + File.separator
							+ fileName;
					filenameCombo.add(fileName);
					String enc = detectEncoding(fileName);
					if (enc != null) {
						encoding.put(fileName, enc);
						selectEncoding(enc);
					}
				}
				filenameCombo.select(0);
			}
			filenames = filenameCombo.getItems().clone();
			DocumentImportWizardPage.this.getContainer().updateButtons();
			String enc = encoding.get(filenameCombo.getText());
			selectEncoding(enc);
		}
	};

	private final SelectionListener encodingSelect = new SelectionListener() {
		public void widgetDefaultSelected(SelectionEvent e) {
		}

		public void widgetSelected(SelectionEvent e) {
			encoding.put(filenameCombo.getText(), encCombo.getText());
		}
	};

	private final SelectionListener fileSelect = new SelectionListener() {
		public void widgetDefaultSelected(SelectionEvent e) {
		}

		public void widgetSelected(SelectionEvent e) {
			String enc = encoding.get(filenameCombo.getText());
			selectEncoding(enc);
		}
	};

	private final ModifyListener fileRename = new ModifyListener() {
		@Override
		public void modifyText(ModifyEvent e) {
			if (filenames == null) {
				String filetext = filenameCombo.getText();
				if (new File(filetext).isFile() ) {
					String enc = detectEncoding(filetext);
					if (enc != null) {
						encoding.put(filetext, enc);
						selectEncoding(enc);
					}
				}
			} 
		}
	};

	public TreeNode getDocumentParent() {
		StructuredSelection selection = (StructuredSelection) docFolder
				.getSelection();
		if (selection.isEmpty())
			return null;

		if (selection.getFirstElement() instanceof DocFolder) {
			DocFolder docFolder = (DocFolder) selection.getFirstElement();
			selectedDB = docFolder.getTreeDB();
			return docFolder;
		}

		if (selection.getFirstElement() instanceof TreeDB) {
			TreeDB treeDB = (TreeDB) selection.getFirstElement();
			selectedDB = treeDB;
			return treeDB.getNode(Document.getTypeRootQId());
		}
		return null;
	}

	public TreeDB getSelectedDB() {
		return selectedDB;
	}

	public String[] getImportFileNames() {
		if (filenames == null) {
			filenames = new String[1];
			filenames[0] = filenameCombo.getText();
			encoding.put(filenames[0], encCombo.getText());
		}
		return filenames;
	}

	public String getEncoding(String filename) {
		String enc = encoding.get(filename);
		if (enc != null)
			return enc;
		else
			return "UTF-8";
	}

	public boolean isDataReady() {
		if (!new File(filenameCombo.getText()).exists())
			return false;
		if (getDocumentParent() == null)
			return false;
		if (getSelectedDB() == null)
			return false;
		return true;
	}

	private String detectEncoding(String pathToFile) {
		FileInputStream in;
		try {
			in = new FileInputStream(pathToFile);
			CharsetDetector detector = new CharsetDetector();
			detector.enableInputFilter(true);
			detector.setText(new BufferedInputStream(in));
			CharsetMatch match = detector.detect();
			in.close();
			return match.getName();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public int selectEncoding(String name) {
		String[] charsetNames = (String[]) Charset.availableCharsets().keySet()
				.toArray(new String[0]);
		for (int i = 0; i < charsetNames.length; i++) {
			if (encCombo.getItem(i).equals(name)) {
				encCombo.select(i);
				return i;
			}
		}
		return -1;
	}
}